/***************************************************************************
*   Copyright (C) 2007 by Messer                                          *
*   neomatraz@gmail.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
/**
	Player window module.
	This code is tested and is stable.
*/
//
#include "mainimpl.h"
#include "playlistitem.h"
#include "slider.h"
#include "equal.h"
#include "playlist.h"
#include "preference.h"
#include "qx_string.h"

//Qt
#include <QtGui>
#include <QtCore>

#define STR_NUM QString::number
#define ITEM list->treeList->topLevelItem(list->treeList->playedItemRow())
#define LIST list->treeList
#define STR_DEC qx_string::decode_string
#define VOID void MainImpl::

static const QString spacing = "  :";
static const QString spacing2 = ":  ";
/** stable code*/
VOID createGUI()
{
	ui.setupUi(this);
	QPalette palette(QColor(54, 54, 54));
	setPalette(palette);
	QPalette palette2(QColor(232, 232, 232));
	QPalette palette3(QColor(90, 90, 90));
	ui.plUp->setPalette(palette2);
	ui.playBt->setPalette(palette2);
	ui.stopBt->setPalette(palette2);
	ui.plDown->setPalette(palette2);
	ui.listBt->setPalette(palette3);
	ui.replayBt->setPalette(palette2);
	ui.eqBt->setPalette(palette3);
	playProgr = new Slider(ui.widgetPr, Qt::Horizontal);
	playProgr->setGeometry(QRect(0, 0, 259, 20));
	PlayVol = new Slider(ui.widgetVol, Qt::Vertical);
	PlayVol->setGeometry(QRect(0, 0, 20, 67));
	PlayVol->setMaximum(100);
	QFont f = ui.label_2->font();
	f.setPointSize(f.pointSize() + 5);
	ui.label_2->setFont(f);
	ui.label_2->setText("-- : --");
}

MainImpl::MainImpl ():
scrollPos(0), playIndex(0), selectedItemDeleted(true)
{
	createGUI();
	list = new PlayList;
	list->readTmpPList();

	eq = new Eq;
	connect(eq, SIGNAL(eqChanged(int, int)), this, SLOT(setEq(int, int)));
	connect(eq, SIGNAL(ampChanged(int)), this, SLOT(setAmp(int)));
	timer = new QTimer(this);
	timer->setInterval (900);
	timer2 = new QTimer(this);
	timer2->setInterval(50);
	scrollTimer = new QTimer(this);
	scrollTimer->setInterval(25);
	scrollTimer->start();

	readSettings();
	list->decodeList();
	setItemsBrush();

	connect (ui.playBt, SIGNAL (clicked()), this, SLOT(playOrPause ()));
	connect (PlayVol, SIGNAL (valueChanged(int)), this, SLOT (setVolume()));
	connect (timer, SIGNAL(timeout()), this, SLOT(getPosition()));
	connect (playProgr, SIGNAL(sliderPressed()), this,
		SLOT(selectValue()));
	connect (playProgr, SIGNAL(valueChanged(int)), this, SLOT(onSlMoved(int)));
	connect (playProgr, SIGNAL(sliderReleased()), this, SLOT(setPosition()));
	connect (ui.stopBt, SIGNAL(clicked()), this, SLOT(stopTrack()));
	connect (timer2, SIGNAL(timeout()), this, SLOT(updateVol()));
	connect (ui.listBt, SIGNAL(toggled(bool)), this, SLOT(showList()));
	connect (list, SIGNAL(listClosed()), this, SLOT(checkListVisible()));
	connect (LIST, SIGNAL(itemActivated(QTreeWidgetItem*, int)), this,
			SLOT(processItem(QTreeWidgetItem*)));
	connect (ui.plUp, SIGNAL(clicked()), this, SLOT(upTrack()));
	connect (ui.plDown, SIGNAL(clicked()), this, SLOT(downTrack()));
	connect (LIST->rmItem, SIGNAL(triggered()), this, SLOT(rmTrack()));
	connect (list, SIGNAL(metaInfoChanged(const QString&)), this,
		SLOT(updateTrackTitle(const QString&)));
	connect (list->treeList->playAct, SIGNAL(triggered()), this,
			SLOT(play_current_item()));
	connect(scrollTimer, SIGNAL(timeout()), this, SLOT(scrollLabel()));

	connect(eq, SIGNAL(closed()), this, SLOT(eqClosed()));
	connect(ui.eqBt, SIGNAL(clicked()), this, SLOT(showEq()));
	connect(list, SIGNAL(listCleared()), this, SLOT(onListCleared()));
	connect(list, SIGNAL(playIndexChanged(int)), this,
		 SLOT(onPIndexChanged(int)));
	connect(LIST, SIGNAL(itemDropped(int)), this, SLOT(onPIndexChanged(int)));
//	connect(LIST, SIGNAL(deletePressed()), this, SLOT(rmTrack()));
	connect(list->prefDialog->reloadBt, SIGNAL(clicked()), this, SLOT(updateEngine()));
//	connect(list->moveUp, SIGNAL(pressed()), this, SLOT(onMoveUpPressed()));
//	connect(list->moveDn, SIGNAL(pressed()), this, SLOT(onMoveDnPressed()));
	connect(LIST, SIGNAL(deletePressed()), this, SLOT(onDelPressed()));
}

VOID onDelPressed()
{
	if(selectedItemDeleted)
		rmTrack();
}

/*VOID onMoveUpPressed()
{

}

VOID onMoveDnPressed()
{

}*/

VOID updateEngine()
{
	stopTrack();
	player->setOutputEngine(list->prefDialog->module->currentText());
}

/** stable code*/
VOID onListCleared()
{
	stopTrack();
	playIndex = 0;
}
/** stable code*/
VOID playOrPause()
{
      if(LIST->topLevelItemCount() != 0)
      {
	switch(player->state())
	{
	case Player::Play:
	ui.playBt->setIcon(QPixmap(":/images/player_play.png"));
	LIST->setPlayedItem(playIndex, false);
	ui.playBt->setChecked(false);
	timer->stop();
	player->pause();
	emit playerState(Player::Pause);
	break;

	case Player::Pause:
	ui.playBt->setIcon(QPixmap(":/images/player_pause.png"));
	ui.playBt->setChecked(true);
	LIST->setPlayedItem(playIndex, true);
	timer->start();
	timer2->start();
	player->play();
	emit playerState(Player::Play);
	break;

	case Player::Idle:
	if(ITEM)
	LIST->setCurrentItem(ITEM);
	else
	LIST->setCurrentItem(LIST->topLevelItem(0));
	processItem(LIST->currentItem());
	emit playerState(Player::Play);
	break;
	}
      }
}
/** stable code*/
VOID onPIndexChanged(int index)
{
	playIndex = index;
	LIST->i = playIndex;
}
/** stable code*/
VOID play_current_item()
{
	if(LIST->topLevelItemCount() != 0 && LIST->currentItem())
	processItem(LIST->currentItem());
}
/** stable code*/
VOID setVolume ()
{
	player->setVol(PlayVol->value());
}
/** stable code*/
VOID setEq(int id , int value)
{
	player->setEQ(id, value);
}
/** stable code*/
VOID setAmp(int value)
{
	player->setAMP(value);
}
/** stable code*/
VOID eqClosed()
{
	ui.eqBt->setChecked(false);
}
/** stable code*/
VOID showEq()
{
	if (ui.eqBt->isChecked())
		eq->show();
	if (!ui.eqBt->isChecked())
		eq->hide();
}
/** stable code*/
VOID getPosition()
{
	callTime();
	if (sec < 10)
	ui.label_2->setText(STR_NUM(min) + " : " + "0" +
			 STR_NUM(sec));
	else
	ui.label_2->setText(STR_NUM(min) +
				 " : " + STR_NUM(sec));
	playProgr->setValue(player->pos());

	if(player->state() != Player::Play)
	{
	stopTrack();


        if (ui.replayBt->isChecked() && ITEM)
	{
		QFile file(dynamic_cast<PlayListItem*>(ITEM)->tags.path);
		if (file.exists())
		{
		player->setTrack(
			dynamic_cast<PlayListItem*>(ITEM)->tags.path);
		LIST->setCurrentItem(ITEM);
		processItem(ITEM);
		title = ((PlayListItem*)ITEM)->tags.title;
		album = ((PlayListItem*)ITEM)->tags.album;
		artist =((PlayListItem*)ITEM)->tags.artist;
		emit trackChanged(artist, album, title);
		return;
		}
	QMessageBox::critical(this, tr("Error!"), tr("File not found!"));
	return;
	}

	if (playIndex + 1 < LIST->topLevelItemCount())
	{
	LIST->setCurrentItem(LIST->topLevelItem(playIndex + 1));
	list->showDir(LIST->currentItem());
	processItem(LIST->currentItem());
	return;
	}
	if (playIndex + 1 == LIST->topLevelItemCount() &&
		list->ui.actionReplayL->isChecked())
	{
	LIST->setCurrentItem(LIST->topLevelItem(0));
	list->showDir(LIST->currentItem());
	processItem(LIST->currentItem());
	return;
	}
	else stopTrack();
	
	return;
	}
}
/** stable code*/
VOID setPosition()
{
	player->setPos(uint(playProgr->value()));
	if(player->state() == Player::Play)
		timer->start();
}
/** stable code*/
VOID selectValue()
{
	if(player->state() == Player::Play)
	timer->stop();
}
/** stable code*/
VOID onSlMoved(int v)
{
	if(playProgr->maximum() != 0)
	{
	uint t = v / 1000;
	uint s = t % 60;
	uint m = (t - s) / 60;
	if(s < 10)
	ui.label_2->setText(STR_NUM(m) + " : 0" + STR_NUM(s));
	else
	ui.label_2->setText(STR_NUM(m) + " : " + STR_NUM(s));
	}
}
/** stable code*/
VOID stopTrack()
{
	setItemsBrush();

	ui.playBt->setIcon(QPixmap(":/images/player_play.png"));
	player->stop();
	timer->stop();
	playProgr->setMaximum(0);
	metaInfo.clear();
	ui.label_2->clear();
	ui.label_2->setText("-- : --");
	ui.label_3->clear();
	ui.label_3->setText("0 kb/s - 0 kHz");
	list->currentPlay->clear();
	emit trackChanged(QString::null, QString::null, QString::null);
	emit playerState(Player::Idle);
}
/** stable code*/
VOID callTime()
{
	uint t = player->pos() / 1000;
	sec = t % 60;
	uint m = t - sec;
	min = m / 60;
}
/** stable code*/
VOID updateVol()
{
	player->setVol(PlayVol->value());
	timer2->stop();
}
/** stable code*/
VOID showList()
{
	if (ui.listBt->isChecked())
		list->show();
	if (!ui.listBt->isChecked())
		list->hide();

}
/** stable code*/
VOID checkListVisible()
{
	ui.listBt->setChecked(false);
}
/** stable code*/
VOID rmTrack()
{
	if(LIST->topLevelItemCount() != 0 && LIST->currentItem())
	{
	selectedItemDeleted = false;
	int row = LIST->indexOfTopLevelItem(LIST->currentItem());
	uint len = dynamic_cast<PlayListItem*>(LIST->currentItem())->tags.length;
		if(row == playIndex)
		{
		stopTrack();
		}
		if(row < playIndex)
		{
		LIST->setPlayedItem(playIndex - 1, true);
		playIndex--;
		}
	LIST->takeTopLevelItem(LIST->indexOfTopLevelItem(LIST->currentItem()));
	for(int m = 0; m<LIST->columnCount(); m++)
		LIST->resizeColumnToContents(m);
	list->totalTime = list->totalTime - len;
	list->updateListInfo();
	if(row == LIST->topLevelItemCount())
	{
		LIST->scrollToItem(LIST->topLevelItem(row - 1),
			QAbstractItemView::PositionAtCenter);
	}
	else
		LIST->scrollToItem(LIST->topLevelItem(row),
			QAbstractItemView::PositionAtCenter);
	setItemsBrush();
	selectedItemDeleted = true;

	if(player->state() == Player::Play)
	{
		LIST->setPlayedItem(playIndex, true);
		return;
	}
	if(player->state() == Player::Pause)
	{
		LIST->setPlayedItem(playIndex, false);
		return;
	}

	}
}
/** stable code*/
VOID setItemsBrush()
{
	for (int a = 0; a<LIST->topLevelItemCount(); a++ )
	{
		for(int m = 0; m<LIST->columnCount(); m++)
		LIST->topLevelItem(a)->setBackground(m, Qt::NoBrush);
	}
}
/** stable code*/
VOID processItem(QTreeWidgetItem* i)
{
	QString track = ((PlayListItem*)i)->tags.path;
	QFile file(track);
	if(!file.exists())
	{
	track.clear();
	QMessageBox::critical(this, tr("Error!"), tr("File not found!"));
	return;
	}
	stopTrack();
	player->setTrack(track);
	LIST->setPlayedItem(LIST->indexOfTopLevelItem(i), true);
	playIndex = LIST->indexOfTopLevelItem(i);

	if(!((PlayListItem*)i)->tags.title.contains(".mp3", Qt::CaseInsensitive)
	&& !((PlayListItem*)i)->tags.title.contains(".ogg", Qt::CaseInsensitive))
	title = STR_DEC(((PlayListItem*)i)->tags.title, list->codec);
	else title = ((PlayListItem*)i)->tags.title;

	album = STR_DEC(((PlayListItem*)i)->tags.album, list->codec);
	artist = STR_DEC(((PlayListItem*)i)->tags.artist, list->codec);
	if(!artist.isEmpty())
	metaInfo = artist + " - " + title;
	else
	metaInfo = title;

	list->currentPlay->setText("<b>" + tr("Playing: ") + "</b>" + metaInfo);
	list->showDir(i);

	int br = ((PlayListItem*)LIST->topLevelItem(playIndex))->tags.bitrate;
	int amp = ((PlayListItem*)LIST->topLevelItem(playIndex))->tags.amp / 1000;
	QString info =  STR_NUM(br) + " kb/s - " + STR_NUM(amp) + " kHz";
	ui.label_3->setText(info);

	emit trackChanged(artist, album, title);
	playTrack();
	scrollPos = 0;
}
/** stable code*/
VOID playTrack()
{
	ui.playBt->setIcon(QPixmap(":/images/player_pause.png"));
	ui.playBt->setChecked(true);
	playProgr->setMaximum(player->length());
	player->setVol(PlayVol->value());
	player->setPos(playProgr->value());
	timer->start();
	timer2->start();
	player->play();
	emit playerState(Player::Play);
	return;
}
/** stable code*/
VOID upTrack()
{
  if(LIST->topLevelItemCount() != 0) {
	if (playIndex != 0)
	LIST->setCurrentItem(LIST->topLevelItem(playIndex - 1));
	else
	LIST->setCurrentItem(LIST->topLevelItem(LIST->topLevelItemCount() - 1));

	list->showDir(LIST->currentItem());
	processItem(LIST->currentItem()); }
}
/** stable code*/
VOID downTrack()
{
  if(LIST->topLevelItemCount() != 0) {
	if (playIndex + 1 != LIST->topLevelItemCount())
	LIST->setCurrentItem(LIST->topLevelItem(playIndex + 1));
	else
	LIST->setCurrentItem(LIST->topLevelItem(0));

	list->showDir(LIST->currentItem());
	processItem(LIST->currentItem()); }
}
/** stable code*/
VOID updateTrackTitle(const QString& t)
{
	metaInfo = t.trimmed();
}

/** stable code*/
VOID keyPressEvent (QKeyEvent *e)
{
	if (e->key() == Qt::Key_Space)
		playOrPause();
}
/** stable code*/
VOID saveSettings()
{
	QSettings settings("messer software" , "qxplayer" );
	settings.setValue("Volume", PlayVol->sliderPosition());
	settings.setValue("PlItem", LIST->playedItemRow());
	settings.setValue("Replay", ui.replayBt->isChecked());
	settings.setValue("ReplayL", list->ui.actionReplayL->isChecked());
	settings.setValue("ListWidth", list->size().width());
	settings.setValue("ListHeight", list->size().height());
	settings.setValue("ListX", list->x());
	settings.setValue("ListY", list->y());
	settings.setValue("PlX", this->x());
	settings.setValue("PlY", this->y());
	settings.setValue("RootDir", list->prefDialog->rootDir());
	settings.setValue("Codec", list->codec);
	settings.setValue("CodecID", list->prefDialog->CSBox->currentIndex());
	settings.setValue("Lang", list->prefDialog->lang->currentText());
	settings.setValue("LangID", list->prefDialog->lang->currentIndex());
///###
//	settings.setValue("Mod", list->prefDialog->module->currentText());
	settings.setValue("ModID", list->prefDialog->module->currentIndex());
///###
	QDir dir(QDir::homePath() + "/.qxplayer");
	if(!dir.exists())
	dir.mkdir(QDir::homePath() + "/.qxplayer");


	QFile::remove(QDir::homePath() + "/.qxplayer/plist.m3u");

	if(LIST->topLevelItemCount() != 0)
	{
	QFile plist(QDir::homePath() + "/.qxplayer/plist.m3u");
	plist.open(QIODevice::WriteOnly);
	QTextStream out(&plist);
	for(int i = 0; i<LIST->topLevelItemCount(); i++)
	out << dynamic_cast<PlayListItem*>(LIST->topLevelItem(i))->tags.path + "\n";
	plist.close();
	}

	eq->save();
}
/** stable code*/
VOID scrollLabel()
{

	uint t = player->length() / 1000;
	uint sec = t % 60;
	uint min = (t - sec) / 60;
	QString time;
	if(player->state() == Player::Play || player->state() == Player::Pause)
	{
	if(sec<10)
	time = " - " + STR_NUM(min) + " : 0" + STR_NUM(sec);
	else
	time = " - " + STR_NUM(min) + " : " + STR_NUM(sec);
	}
	else
	time = QString::null;

	QFontMetrics fm(ui.label->font());
	QString tmp1 = metaInfo;
	QString tmp2 = tr("Welcome to qxPlayer!");
	QString current;
	if(!metaInfo.isEmpty())
	{
	current = tmp1.append(time).append(spacing);
	current = spacing2 + current;
	}
	else
	{
	current = tmp2.append(time).append(spacing);
	current = spacing2 + current;
	}


	QPixmap tmpPxm(ui.label->width(), fm.height());
	scrollPm = tmpPxm;
	scrollPm.fill(QColor(54, 54, 54));
	QFont font = ui.label->font();
	font.setBold(true);
	QPainter p(&scrollPm);
	p.setFont(font);
	p.setPen(QColor("white"));
	int w = fm.width(current);

	if(w<1) return;

	int x = -scrollPos;

	while(x<scrollPm.width())
	{
	p.drawText(QPoint(x, ui.label->height() / 2), current);
	x += w;
	}
	++scrollPos;
	if(scrollPos >= w) scrollPos = 0;


	p.end();
	ui.label->setPixmap(scrollPm);

}
/** stable code*/
VOID readSettings()
{
	QSettings settings("messer software" , "qxplayer" );
	int vol = settings.value("Volume").toInt();
	if(vol != 0)
		PlayVol->setValue(vol);
	else
		PlayVol->setValue(PlayVol->maximum());
	int pIt = settings.value("PlItem").toInt();
	playIndex = pIt;
	LIST->setPlayedItem(pIt, false);
	LIST->setCurrentItem(LIST->topLevelItem(pIt));
	bool checked = settings.value("Replay").toBool();
	ui.replayBt->setChecked(checked);
	list->ui.actionReplayL->setChecked(settings.value("ReplayL").toBool());
	int x = settings.value("ListX").toInt();
	int y = settings.value("ListY").toInt();
	int lW = settings.value("ListWidth").toInt();
	int lH = settings.value("ListHeight").toInt();
	int px = settings.value("PlX").toInt();
	int py = settings.value("PlY").toInt();
	list->codec = settings.value("Codec").toString();
	list->prefDialog->CSBox->setCurrentIndex(settings.value("CodecID").toInt());
	list->prefDialog->lang->setCurrentIndex(settings.value("LangID").toInt());
	QString bD = settings.value("RootDir").toString();
	if(list->browDir.isEmpty())
		list->browDir = bD;

	list->prefDialog->root_dir_e->setText(bD);
	list->prefDialog->module->setCurrentIndex(settings.value("ModID").toInt());
//	player->setOutputEngine(list->prefDialog->module->currentText());
	player = new Player(this, list->prefDialog->module->currentText());

	list->setGeometry(QRect(x, y, lW, lH));
	this->setGeometry(QRect(px, py, width(), height()));
	eq->read();
}

