/***************************************************************************
*   Copyright (C) 2007 by Messer                                          *
*   neomatraz@gmail.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "player.h"
#include <QtGui>

//constructor
Player::Player(QObject* parent, QString eng)
: QObject(parent),
position(0)
{
	m_xine = xine_new();
	if (!m_xine)
		return;
	xine_init(m_xine);
	const char * _eng = (const char*)eng.toLocal8Bit();
	m_port = xine_open_audio_driver(m_xine, _eng, NULL);
	m_stream = xine_stream_new (m_xine, m_port, NULL);
	if(!m_stream)
	return;
	xine_set_param( m_stream, XINE_PARAM_IGNORE_VIDEO, 1 );
}


//destructor
Player::~Player()
{
	pause();
	xine_set_param(m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
	xine_set_param(m_stream, XINE_PARAM_AUDIO_MUTE, 0);
	xine_set_param (m_stream, XINE_PARAM_AUDIO_VOLUME, 99);
	xine_stop(m_stream);
	xine_close_audio_driver (m_xine, m_port);
	xine_exit(m_xine);
}

/// functions implementation.

uint Player::pos()
{
	int time = 0;
	int len;
	xine_get_pos_length (m_stream, NULL, &time, &len);
	position = time;
	return time;
}

uint Player::length()
{
	int time;
	int len = 0;
	xine_get_pos_length (m_stream, NULL, &time, &len);
	return len;
}

bool Player::isRunning()
{
	if(xine_get_status(m_stream) == XINE_STATUS_PLAY &&
		xine_get_param(m_stream, XINE_PARAM_SPEED) == XINE_SPEED_NORMAL)
	return true;
	return false;
}

bool Player::isPaused()
{
	if(xine_get_status(m_stream) == XINE_STATUS_PLAY &&
		xine_get_param(m_stream, XINE_PARAM_SPEED) == XINE_SPEED_PAUSE)
	return true;
	return false;
}

/*bool Player::isMute() const
{
	if (xine_get_param(m_stream, XINE_PARAM_AUDIO_MUTE) == 1)
	return true;
	return false;
}*/

/// slots implementation.

void Player::setOutputEngine(const QString& n)
{
	const char* eng = static_cast<const char*>(n.toLocal8Bit());
	if(isRunning() || isPaused())
		stop();
	else xine_stop(m_stream);
	xine_close_audio_driver (m_xine, m_port);
	m_port = xine_open_audio_driver(m_xine, eng, NULL);
	m_stream = xine_stream_new (m_xine, m_port, NULL);
	xine_set_param( m_stream, XINE_PARAM_IGNORE_VIDEO, 1 );
}

void Player::setVol(int vol)
{
	volume = vol;
	if(isRunning())
	{
	xine_set_param (m_stream, XINE_PARAM_AUDIO_VOLUME, vol);
	return;
	}
	else
	xine_set_param (m_stream, XINE_PARAM_AUDIO_VOLUME, 99);
}

void Player::setPos(uint pos)
{
	position = pos;
	if(isRunning())
		xine_play(m_stream, 0, position);
}

void Player::setTrack (const QString& t)
{
	if (t == track) return;
	else
	{
	track = t;
	xine_open (m_stream, (const char*)track.toLocal8Bit());
	}
}

void Player::play()
{
	if(xine_get_param(m_stream, XINE_PARAM_SPEED) == XINE_SPEED_PAUSE)
	{
	xine_set_param(m_stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
	xine_play(m_stream, 0, position);
	return;
	}
	else
	xine_play (m_stream, 0, 0);
}

void Player::stop()
{
        xine_set_param( m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
	xine_stop(m_stream);
	track.clear();
	position = 0;
}

void Player::pause()
{
	if(xine_get_status(m_stream) == XINE_STATUS_PLAY)
	{
	xine_set_param (m_stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
        xine_set_param( m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
	}
}

#define PLAYER_MUTED 1
#define PLAYER_NOT_MUTED 0

void Player::mute()
{
	switch (xine_get_param(m_stream, XINE_PARAM_AUDIO_MUTE))
	{
	case PLAYER_MUTED:
		xine_set_param(m_stream, XINE_PARAM_AUDIO_MUTE, 0);
		break;
	case PLAYER_NOT_MUTED:
		xine_set_param(m_stream, XINE_PARAM_AUDIO_MUTE, 1);
		break;
	}
}

void Player::setEQ(int slId, int val)
{
	switch(slId)
	{
	case 1:
	xine_set_param(m_stream, XINE_PARAM_EQ_30HZ, val);
		break;
	case 2:
	xine_set_param(m_stream, XINE_PARAM_EQ_60HZ, val);
		break;
	case 3:
	xine_set_param(m_stream, XINE_PARAM_EQ_125HZ, val);
		break;
	case 4:
	xine_set_param(m_stream, XINE_PARAM_EQ_250HZ, val);
		break;
	case 5:
	xine_set_param(m_stream, XINE_PARAM_EQ_500HZ, val);
		break;
	case 6:
	xine_set_param(m_stream, XINE_PARAM_EQ_1000HZ, val);
		break;
	case 7:
	xine_set_param(m_stream, XINE_PARAM_EQ_2000HZ, val);
		break;
	case 8:
	xine_set_param(m_stream, XINE_PARAM_EQ_4000HZ, val);
		break;
	case 9:
	xine_set_param(m_stream, XINE_PARAM_EQ_8000HZ, val);
		break;
	case 10:
	xine_set_param(m_stream, XINE_PARAM_EQ_16000HZ, val);
		break;
	}
}

void Player::setAMP(int v)
{
	xine_set_param(m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, v);
}

Player::State Player::state()
{
	if(isRunning())
		return Player::Play;
	if(isPaused())
		return Player::Pause;
	return Player::Idle;
}
