/***************************************************************************
*   Copyright (C) 2007 by Messer                                          *
*   neomatraz@gmail.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef _PLAYER_H_
#define _PLAYER_H_

#include <QObject>

#include <xine.h>


class Player : public QObject
{
	public:

		enum State {
		Pause = 0x01,
		Play = 0x02,
		Idle = 0x04
		};


		Player(QObject * parent = 0, QString eng = "auto");
		~Player();

		State state();

// Returns a current position in milliseconds
		uint pos ();

// Returns a current vulume level ( 0...99)
		inline int vol() { return volume; }

// Returns a track size in milliseconds
		uint length ();

// Is volume muted?
//		bool isMute() const;



	public slots:

// Set volume to 0...99
		void setVol (int vol = 0);

// Set position to (in milliseconds)
		void setPos (uint pos = 0);

// File path
		void setTrack (const QString& t);

// Play
		void play();

// Stop playing, reset progress and length
		void stop();

// Pause
		void pause();

// Mute/unmute the sound
		void mute();

		void setEQ(int slId, int value);

		void setAMP(int v);

		void setOutputEngine(const QString& name);


	private:
		uint position;
		int volume;
		QString track;
		bool isRunning();
		bool isPaused();

// XINE variables initialization
		xine_t * m_xine;
		xine_audio_port_t * m_port;
		xine_stream_t * m_stream;
};
#endif
