/***************************************************************************
*   Copyright (C) 2007 by Messer                                          *
*   neomatraz@gmail.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

/**
	Playlist module
*/
//player includes
#include "playlist.h"
#include "playlistwidget.h"
#include "playlistitem.h"
#include "ui_about.h"
#include "lineedit.h"
#include "preference.h"
//Qt includes
#include <QtGui>
#include <QtCore>
#include <QtNetwork>
//Taglib includes
#include <tag.h>
#include <fileref.h>
#include <taglib/id3v1genres.h>
//c++ includes
#include <string>

#define STR_DEC qx_string::decode_string
#define GET_TIME qx_string::get_timestring

PlayList::PlayList() : totalTime(0), http(0)
{
	ui.setupUi(this);
	treeList = new PlayListWidget(ui.pList);
	QVBoxLayout *l = new QVBoxLayout;
	l->setMargin(1);
	l->setSpacing(2);
	l->addWidget(createSearchWidget());
	l->addWidget(treeList);
	ui.pList->setLayout(l);
	treeList->setFocusPolicy(Qt::StrongFocus);
	lab = new QLabel(this);
	currentPlay = new QLabel(this);
	ui.statusBar->insertWidget(0, lab);
	ui.statusBar->insertWidget(1, currentPlay);
	buff = new QBuffer;
	prefDialog = new preference(this);
	treeList->setFocus();
	moveUpT = new QTimer(this);
	moveDnT = new QTimer(this);

	m_xine = xine_new();
	xine_init(m_xine);
	m_port = xine_open_audio_driver(m_xine, "auto", NULL);
	m_stream = xine_stream_new (m_xine, m_port, NULL);

	connect(moveUpT, SIGNAL(timeout()), this, SLOT(onMoveUpT()));
	connect(moveDnT, SIGNAL(timeout()), this, SLOT(onMoveDnT()));

	connect (ui.prefsAct, SIGNAL(triggered()), this, SLOT(showPrefsWindow()));
	connect (treeList->addAct, SIGNAL(triggered()),
		 this, SLOT(addTrack()));
	connect (ui.actionAddF, SIGNAL(triggered()),
		 this, SLOT(addTrack()));
	connect(treeList, SIGNAL(itemClicked(QTreeWidgetItem*, int)),
		 this, SLOT(showDir(QTreeWidgetItem*)));
	connect(treeList,
	 SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
		 this, SLOT(showDir(QTreeWidgetItem*)));
	connect(treeList->metaInfoAct, SIGNAL(triggered()), this,
			SLOT(getMetaInfo()));
	connect(ui.actionSave, SIGNAL(triggered()), this, SLOT(
		savePlayList()));
	connect(ui.actionAddL, SIGNAL(triggered()), this, SLOT(
		openPlayList()));
	connect(treeList->addListAct, SIGNAL(triggered()), this, SLOT(
		openPlayList()));
	connect(ui.actionAboutQt, SIGNAL(triggered()), this, SLOT(
		aboutQt()));
	connect(ui.actionAbout, SIGNAL(triggered()), this, SLOT(
		about()));
	connect(ui.actionClear, SIGNAL(triggered()), this, SLOT(
		clearList()));
	connect(treeList->clearAct, SIGNAL(triggered()), this, SLOT(clearList()));

	connect(searchL, SIGNAL(textChanged(const QString&)), this,
		SLOT(search(const QString&)));
	connect(ui.tabWidget, SIGNAL(currentChanged(int)), this,
	 SLOT(getLyrics(int)));
	connect(moveUp, SIGNAL(released()), this, SLOT(onMoveUpReleased()));
	connect(moveDn, SIGNAL(released()), this, SLOT(onMoveDnReleased()));
	connect(moveUp, SIGNAL(pressed()), this, SLOT(onMoveUpPressed()));
	connect(moveDn, SIGNAL(pressed()), this, SLOT(onMoveDnPressed()));
	connect(clBt, SIGNAL(clicked()), this, SLOT(clearSearch()));
	connect(ui.AddDirAct, SIGNAL(triggered()), this, SLOT(addDir()));
	connect(prefDialog, SIGNAL(settingsSaved(const QString&)), this,
		SLOT(savePrefs(const QString&)));
	connect(prefDialog, SIGNAL(setCodec(const QString&)), this,
		SLOT(set_codec(const QString&)));
	connect(ui.actTCtl, SIGNAL(triggered()), this, SLOT(toolBarCtl()));
}

void PlayList::onMoveUpT()
{
	onMoveUp();
}

void PlayList::onMoveDnT()
{
	onMoveDn();
}

void PlayList::onMoveDnReleased()
{
	moveDnT->stop();
}

void PlayList::onMoveUpReleased()
{
	moveUpT->stop();
}

void PlayList::onMoveUpPressed()
{
	onMoveUp();
	moveUpT->start(200);
}

void PlayList::onMoveDnPressed()
{
	onMoveDn();
	moveDnT->start(200);
}

void PlayList::toolBarCtl()
{
	if(ui.toolBar->isVisible())
		ui.toolBar->hide();
	else ui.toolBar->show();
}

QWidget *  PlayList::createSearchWidget()
{
	QWidget *w = new QWidget(this);
	w->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	moveUp = new QToolButton(w);
	moveUp->setIcon(QPixmap(":/images/1uparrow.png"));
	moveUp->setMaximumSize(QSize(18, 18));
	moveUp->setToolTip(tr("Move selected item up"));
	moveDn = new QToolButton(w);
	moveDn->setIcon(QPixmap(":/images/1downarrow.png"));
	moveDn->setMaximumSize(QSize(18, 18));
	moveDn->setToolTip(tr("Move selected item down"));
	clBt = new QToolButton(w);
	clBt->setIcon(QPixmap(":/images/clbt.png"));
	clBt->setMaximumSize(QSize(18, 18));
	moveUp->setFocusPolicy(Qt::NoFocus);
	moveDn->setFocusPolicy(Qt::NoFocus);
	clBt->setFocusPolicy(Qt::NoFocus);
	searchL = new LineEdit(w);
	QLabel *l = new QLabel(tr("  Search:"), w);
	l->setMaximumSize(QSize(100,20));
	l->setBuddy(searchL);
	QHBoxLayout *lay = new QHBoxLayout(w);
	lay->setMargin(0);
	lay->setSpacing(2);
	lay->addWidget(moveUp);
	lay->addWidget(moveDn);
	lay->addWidget(clBt);
	lay->addWidget(l);
	lay->addWidget(searchL);
	w->setLayout(lay);
	return w;
}

void PlayList::clearSearch()
{
	searchL->clear();
}

void PlayList::clearList()
{
	treeList->clear();
	treeList->i = 0;
	browDir = prefDialog->rootDir();
	totalTime = 0;
	updateListInfo();
	emit listCleared();
}

void PlayList::onMoveUp()
{
	if(treeList->topLevelItemCount() != 0 && treeList->indexOfTopLevelItem(treeList->currentItem()) != 0)
	{
		PlayListItem *__it = dynamic_cast<PlayListItem*>(treeList->currentItem());
		int r = treeList->indexOfTopLevelItem(__it);
		PlayListItem *it = new PlayListItem(NULL);
		it->tags = __it->tags;


		for(int a = 0; a<treeList->columnCount(); a++)
		{
			it->setBackground(a, __it->background(a));
			it->setText(a, __it->text(a));
		}

		it->setTextAlignment(3, 0x0002);
		it->setTextAlignment(4, 0x0002);
		treeList->takeTopLevelItem(r);

		treeList->insertTopLevelItem(r - 1, it);
		treeList->scrollToItem(it, QAbstractItemView::PositionAtCenter);
		treeList->setCurrentItem(it);
		if(r == treeList->i)
		{
			emit playIndexChanged(treeList->indexOfTopLevelItem(it));
			__it = NULL;
			return;
		}

		if(r == treeList->i + 1)
		{
			emit playIndexChanged(treeList->i + 1);
		}

		__it = NULL;
	}
}

void PlayList::onMoveDn()
{
	if(treeList->topLevelItemCount() != 0 && treeList->indexOfTopLevelItem(treeList->currentItem()) !=
	 treeList->topLevelItemCount() - 1)
	{
		PlayListItem *__it = dynamic_cast<PlayListItem*>(treeList->currentItem());
		int r = treeList->indexOfTopLevelItem(__it);
		PlayListItem *it = new PlayListItem(NULL);
		it->tags = __it->tags;

		for(int a = 0; a<treeList->columnCount(); a++)
		{
			it->setBackground(a, __it->background(a));
			it->setText(a, __it->text(a));
		}

		it->setTextAlignment(3, 0x0002);
		it->setTextAlignment(4, 0x0002);

		treeList->takeTopLevelItem(r);

		treeList->insertTopLevelItem(r + 1, it);
		treeList->scrollToItem(it, QAbstractItemView::PositionAtCenter);
		treeList->setCurrentItem(it);

		if(r == treeList->i)
		{
			emit playIndexChanged(treeList->indexOfTopLevelItem(it));
			__it = NULL;
			return;
		}

		if(r == treeList->i - 1)
		{
			emit playIndexChanged(treeList->i - 1);
		}
		__it = NULL;
	}
}

#define DEC_ITEM dynamic_cast<PlayListItem*>(treeList->topLevelItem(i))->tags
#define CODEC (const char*)codec.toLocal8Bit()

void PlayList::decodeList()
{
if(treeList->topLevelItemCount() != 0)
	for(int i = 0; i<treeList->topLevelItemCount(); i++)
	{
	  if(!DEC_ITEM.title.isEmpty() &&
	 !DEC_ITEM.title.contains(".mp3", Qt::CaseInsensitive)
	  && !DEC_ITEM.title.contains(".ogg", Qt::CaseInsensitive))
	    treeList->topLevelItem(i)->setText(0, STR_DEC(DEC_ITEM.title,
		 CODEC));
	  if(!DEC_ITEM.artist.isEmpty())
	    treeList->topLevelItem(i)->setText(1, STR_DEC(DEC_ITEM.artist,
		 CODEC));
	  if(!DEC_ITEM.album.isEmpty())
	    treeList->topLevelItem(i)->setText(2, STR_DEC(DEC_ITEM.album,
		 CODEC));
	}
}

void PlayList::set_codec(const QString& s)
{
	codec = s;
	decodeList();
}

void PlayList::readTmpPList()
{
	QFile fin(QDir::homePath() + "/.qxplayer/plist.m3u");
	if(!fin.exists())
		return;

	fin.open(QIODevice::ReadOnly);
	QTextStream in(&fin);

	while(!in.atEnd())
	{
		fileName = in.readLine().trimmed();
		if(!fileName.contains("#"))
			processTrack(fileName);
		else fileName.clear();
	}

	fin.close();
	showDir(treeList->currentItem());
}

void PlayList::openPlayList()
{
	QString name = QFileDialog::getOpenFileName(
			this,
			tr("Open list"),
			QString::null,
			"*.m3u");
	if(name.isEmpty())
		return;

	QFile playList(name);
	playList.open(QIODevice::ReadOnly);

	QTextStream in(&playList);

	while(!in.atEnd())
	{
		fileName = in.readLine().trimmed();
		if(!fileName.contains("#"))
			processTrack(fileName);
		else fileName.clear();
	}
	playList.close();
	showDir(treeList->currentItem());

}

void PlayList::closeEvent(QCloseEvent*)
{
	emit listClosed();
}

void PlayList::addTrack()
{
	fileName.clear();
	fileName = QFileDialog::getOpenFileName(this,
				tr("Select track"),
				browDir,
				QString("*.mp3 , *.ogg , *.wav , *.wma "));
	if (fileName.isEmpty())
		return;

	processTrack(fileName);


}

void PlayList::showDir(QTreeWidgetItem* it)
{
	if(it)
	{
	QString p = ((PlayListItem*)it)->tags.path;
	browDir = p.left(p.lastIndexOf('/'));
	}
}

void PlayList::getMetaInfo()
{
#define TRACK dynamic_cast<PlayListItem*>(treeList->currentItem())->tags.path
#define CI Qt::CaseInsensitive
	if (treeList->topLevelItemCount() != 0 && TRACK.contains(".mp3", CI)
		|| treeList->topLevelItemCount() != 0 && TRACK.contains(".ogg", CI))
	{
		QFile file(TRACK);
		if (!file.exists())
			return;
		QDialog *meta = new QDialog;
		uim.setupUi(meta);
		TagLib::FileRef f((const char*)TRACK.toLocal8Bit());
		QString art = STR_DEC(TStringToQString(f.tag()->artist()), CODEC);
		QString tit = STR_DEC(TStringToQString(f.tag()->title()), CODEC);
		QString alb = STR_DEC(TStringToQString(f.tag()->album()), CODEC);
		uint year = f.tag()->year();
		QString gen = TStringToQString(f.tag()->genre());
		QString btr = QString::number(f.audioProperties()->bitrate()) + tr(" kb/s");
		QStringList list;
		TagLib::StringList genres = TagLib::ID3v1::genreList();

		for( TagLib::StringList::ConstIterator it = genres.begin(), end =
			genres.end(); it != end; ++it )
			list += TStringToQString( (*it) );

		int l = f.audioProperties()->length();

		uim.artist->setText(art.trimmed());
		uim.title->setText(tit.trimmed());
		uim.album->setText(alb.trimmed());
		uim.year->setValue(year);
		for (int a = 0; a<list.size(); a++)
			uim.genre->addItem(list[a].trimmed());
		if(gen.trimmed().isEmpty())
		{
			uim.genre->insertItem(0, "");
			uim.genre->setCurrentIndex(0);
		}
		else
		{
			uim.genre->insertItem(0, gen);
			uim.genre->setCurrentIndex(0);
		}
		uim.bits->setText(btr);
		uim.length->setText(GET_TIME(int(l*1000)));
		uim.path->setText(dynamic_cast<PlayListItem*>(
			treeList->currentItem())->tags.path);
		connect(uim.saveBt, SIGNAL(clicked()), this, SLOT(saveMetaTags()));
		meta->exec();
		delete meta;
	} else {
	QMessageBox::information(this, tr("Error!"), tr("Meta-tags in this sound format is not supported."));
	}
}

void PlayList::saveMetaTags()
{
	std::string art = (const char*)uim.artist->text().toLocal8Bit();
	std::string tit = (const char*)uim.title->text().toLocal8Bit();
	std::string alb = (const char*)uim.album->text().toLocal8Bit();
	std::string gen = (const char*)uim.genre->currentText().toLocal8Bit();

	TagLib::FileRef f((const char*)TRACK.toLocal8Bit());
	f.tag()->setArtist(art);
	f.tag()->setTitle(tit);
	f.tag()->setAlbum(alb);
	f.tag()->setYear(uim.year->value());
	f.tag()->setGenre(gen);
	if(!f.save())
	{
		QMessageBox::warning(this, tr("Error!"), tr("Cannot save meta-data"));
		return;
	}

	QString metaInfo;

	if(!uim.title->text().trimmed().isEmpty())
	{
		metaInfo = uim.artist->text() + " - " + uim.title->text();
		if(treeList->indexOfTopLevelItem(treeList->currentItem()) ==
		 treeList->playedItemRow())
			emit metaInfoChanged(metaInfo);


#define TAGS ((PlayListItem*)treeList->currentItem())->tags
		TAGS.artist = uim.artist->text();
		TAGS.title = uim.title->text();
		TAGS.album = uim.album->text();
		((PlayListItem*)treeList->currentItem())->tags.year =
		 uim.year->value();

		treeList->currentItem()->setText(0, TAGS.title);
		treeList->currentItem()->setText(1, TAGS.artist);
		treeList->currentItem()->setText(2, TAGS.album);
#undef TAGS
	}
}


void PlayList::savePlayList()
{
	if(treeList->topLevelItemCount() != 0)
	{
		QString name = QFileDialog::getSaveFileName(
				this,
				tr("Save playlist"),
				browDir,
				"*.m3u") + ".m3u";
		if (name.isEmpty())
			return;
		QFile playList(name);
		playList.open(QIODevice::WriteOnly);
		QTextStream out(&playList);
		out << "#EXTM3U\n";
#define ITEM ((PlayListItem*)treeList->topLevelItem(i))
		for ( int i = 0; i<treeList->topLevelItemCount(); i++)
			out << ITEM->tags.path + "\n"/*).toUtf8()*/;
		playList.close();
	}
#undef ITEM
}

void PlayList::processTrack(const QString& path)
{
	QFile file(path);
	if(!file.exists())
	{
		fileName.clear();
		QMessageBox::critical(this, tr("Error!"), tr("File not found!"));
		return;
	}

	PlayListItem *i = new PlayListItem(treeList);

#define PL_ITEM treeList->topLevelItem(treeList->i)
	if(treeList->topLevelItemCount() != 0)
	{
		QBrush tmpBr = PL_ITEM->background(0);
		for(int m = 0; m<treeList->columnCount(); m++)
			PL_ITEM->setBackground(m, tmpBr);
	}
#undef PL_ITEM

	i->tags.path = path;
	treeList->setCurrentItem(i);
	showDir(i);

/// MP3 & OGG files processing
	if(path.contains(".mp3", CI) || path.contains(".ogg", CI))
	{
	QString artist, title, album;
	TagLib::FileRef f((const char*)path.toLocal8Bit());
	artist = TStringToQString(f.tag()->artist()).trimmed();
	title = TStringToQString(f.tag()->title()).trimmed();
	album = TStringToQString(f.tag()->album()).trimmed();
	totalTime = totalTime + f.audioProperties()->length();
	updateListInfo();
	uint t = f.audioProperties()->length();

	i->setText(3, GET_TIME(int(t*1000)));
	i->setText(4, QString::number(f.audioProperties()->bitrate()) + tr(" kb/s"));
	i->setTextAlignment(3, 0x0002);
	i->setTextAlignment(4, 0x0002);
	i->tags.length = t;
	i->tags.bitrate = f.audioProperties()->bitrate();
	i->tags.amp = f.audioProperties()->sampleRate();
	
		if (title.isEmpty() || artist.isEmpty() && title.isEmpty())
		{
			QString t1 = path.right(path.length()
			 - path.lastIndexOf('/'));
			i->setText(0, t1.right(t1.length() - 1));
			i->tags.title = t1.right(t1.length() - 1);
			for(int m = 0; m<treeList->columnCount(); m++)
				treeList->resizeColumnToContents(m);
			return;
		}

	i->tags.title = title;
	i->tags.album = album;
	i->tags.artist = artist;
	i->tags.year = f.tag()->year();
	i->setText(0,STR_DEC(title,CODEC));
	i->setText(1, STR_DEC(artist,CODEC));
	i->setText(2, STR_DEC(album,CODEC));
	for(int m = 0; m<treeList->columnCount(); m++)
		treeList->resizeColumnToContents(m);
	return;
#undef CI
	}

/// We have any sound formats, don't containig meta-tags
	QString title = path.right(path.length() - path.lastIndexOf('/'));
	i->setText(0, title.right(title.length() - 1));
	xine_open(m_stream, ((const char*)path.toLocal8Bit()));
	int length = 0;
	xine_get_pos_length(m_stream, NULL, NULL, &length);
	i->setText(3, GET_TIME(length));
	i->tags.length = uint(length);
	i->tags.title = title.right(title.length() - 1);
	i->tags.bitrate = int(xine_get_stream_info(m_stream,
	 XINE_STREAM_INFO_BITRATE));
	i->tags.amp =
		xine_get_stream_info(m_stream,XINE_STREAM_INFO_AUDIO_SAMPLERATE);
	i->setText(4, QString::number(i->tags.bitrate) + tr(" kb/s"));
	i->setTextAlignment(3, 0x0002);
	i->setTextAlignment(4, 0x0002);

	for(int m = 0; m<treeList->columnCount(); m++)
		treeList->resizeColumnToContents(m);
	xine_stop(m_stream);
}

void PlayList::search(const QString& key)
{
#define CURR_IT treeList->topLevelItem(i)
	for(int i = 0; i<treeList->topLevelItemCount(); i++)
	{
		treeList->topLevelItem(i)->setHidden(false);
	}
	for(int i = 0; i<treeList->topLevelItemCount(); i++)
	{
		QString tmp;
		tmp.append(CURR_IT->text(0) + " ");
		tmp.append(CURR_IT->text(1) + " ");
		tmp.append(CURR_IT->text(2));
		if(!tmp.contains(key, Qt::CaseInsensitive))
		CURR_IT->setHidden(true);
	}
}

void PlayList::getLyrics(int ind)
{
	ui.textEdit->clear();
	buff->reset();
//	if(http) http->deleteLater();
  if(treeList->topLevelItemCount() != 0 && treeList->currentItem())
  {
	QString art =
	 dynamic_cast<PlayListItem*>(treeList->currentItem())->tags.artist;
	QString tit =
	 dynamic_cast<PlayListItem*>(treeList->currentItem())->tags.title;
	if(tit.isEmpty() || art.isEmpty())
	{
		ui.textEdit->setText(tr("No lyrics found!"));
		return;
	}
	if(ind == 1)
	{
		http = new QHttp(this);
		connect(http, SIGNAL(done(bool)), this, SLOT(showLyrics(bool)));
		http->setHost("www.autolyrics.com");
		QString p = "/lyrics/for/" + tit + "/by/" + art;
		QUrl url(p);
		http->get(QString(url.toEncoded()), buff);
	}
  }
}

void PlayList::showLyrics(bool error)
{
	if(!error)
	{
		PlayListItem *item  = dynamic_cast<PlayListItem*>(treeList->currentItem());
		QString key = "td width=" + QString::number(245);
		key.append('"');
		key.insert(9, '"');
		ui.textEdit->clear();
		QString tmp = QString(buff->data());
		tmp = tmp.right(tmp.length() - tmp.indexOf(key));
		tmp = tmp.left(tmp.indexOf("</td>"));
		tmp = tmp.right(tmp.length() - 42);
		tmp.insert(0, "<b>" + item->tags.artist + " - " + item->tags.title+ "</b><br>");
		ui.textEdit->setHtml(tmp);
	}
	else
		ui.textEdit->setText(tr("No lyrics found!"));

	http=0;
	http->deleteLater();
}

void PlayList::addDir()
{
	fileName.clear();
	fileName = QFileDialog::getExistingDirectory(this,
				tr("Select directory"),
				browDir);
	if(fileName.isEmpty())
		return;

	QStringList tracks_list;
	tracks_list = scanDir(fileName);
	for (int i = 0; i < tracks_list.count(); i++)
		processTrack(tracks_list[i]);
}

QStringList PlayList::scanDir(const QString& dirName)
{
	QStringList total, tmp_list;
	QDir tmp_dir(dirName);
	total << tmp_dir.entryList(QStringList() << "*.mp3" << "*.ogg" << "*.wav"
	<< "*.wma"
	, QDir::Files, QDir::Name);

	for(int i = 0; i < total.size(); i++)
		total[i] = QString("%1/%2").arg(dirName).arg(total[i]);

	tmp_list = tmp_dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot, QDir::Name);
	for(int i = 0; i < tmp_list.size(); i++)
		total << scanDir(QString("%1/%2").arg(dirName).arg(tmp_list[i]));

	return total;
}

void PlayList::updateListInfo()
{
	int t = totalTime;
	int sec = t % 60;

	int min = (t - sec) / 60;
	int h = min / 60;
	min = min - h * 60;

	QString _min = QString::number(min);
	if(min<10)
	_min = "0" + _min;
	

	QString _totalTime;
	if(sec<10)
		_totalTime = QString::number(h) + " : " + _min + " : 0" +
			 QString::number(sec);
	else
		_totalTime = QString::number(h) + " : " + _min + " : " +
		 QString::number(sec);

	QString _count = tr("Tracks: ") +
	 QString::number(treeList->topLevelItemCount());
	lab->setText(_count + " - " + _totalTime );
}

void PlayList::selectAll()
{
	searchL->setSelection(0, searchL->text().length());
}

void PlayList::showPrefsWindow()
{
	if(!prefDialog->exec())
		return;
}

void PlayList::savePrefs(const QString& s)
{
	if(browDir.isEmpty())
		browDir = s;
}

void PlayList::aboutQt()
{
	QMessageBox::aboutQt(this);
}

void PlayList::about()
{
	QDialog *d = new QDialog;
	Ui::About about;
	about.setupUi(d);
	d->exec();
	delete d;
}
