/***************************************************************************
*   Copyright (C) 2007 by Messer                                          *
*   neomatraz@gmail.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "slider.h"

#define LINE_HEIGHT 6

Slider::Slider(QWidget *parent, Qt::Orientation o) : QAbstractSlider(parent)
{
	setOrientation(o);
	setMaximum(0);
	if(o == Qt::Horizontal)
		setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	else
		setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
}

void Slider::paintEvent(QPaintEvent*)
{
	const int w   = orientation() == Qt::Horizontal ? width() : height();
	int pos = int( double(w) / (maximum() + 1) * sliderPosition() );
	QPainter p(this );

	p.setPen(QColor("darkCyan"));

	if(orientation() == Qt::Horizontal)
	{
	p.drawRect(QRectF(0, (height() / 2) - 3, width() - 1, LINE_HEIGHT));
	p.setBrush(QBrush(QColor(39, 64, 139), Qt::SolidPattern));
	p.drawRect(QRectF(0, (height() / 2) - 3, pos, LINE_HEIGHT));
	
	}
	else
	{
	p.drawRect(QRectF((width() / 2) - 3, 0, LINE_HEIGHT, height()));
	p.setBrush(QBrush(QColor(39, 64, 139), Qt::SolidPattern));
	p.drawRect(QRectF((width() / 2) - 3, height() - 1,  LINE_HEIGHT, -pos));

	}

	if ( orientation() == Qt::Vertical )
	{
	p.translate( 0, height() );
	p.rotate( -90 );
	pos = pos + 1;
	}
	QPolygon pa( 3 );
	pa.setPoint(0, (pos) - 3, 5);
	pa.setPoint(1, (pos), 8);
	pa.setPoint(2, (pos) + 3, 5);
	p.setPen(QColor(135, 206, 255));
	p.setBrush(QBrush(QColor("cyan"), Qt::SolidPattern));
	p.drawConvexPolygon( pa );

	p.end();
}

void Slider::mousePressEvent(QMouseEvent* e)
{
	QAbstractSlider::mousePressEvent(e);
	if(orientation() == Qt::Horizontal)
	setValue(int(maximum() / double(width()) * e->x()) );
	else
	setValue(maximum() - int(maximum() / double(height()) * e->y()) );
	emit sliderPressed();

}

void Slider::mouseReleaseEvent(QMouseEvent* e)
{
	QAbstractSlider::mouseReleaseEvent(e);
	emit sliderReleased();
	emit valueChanged(value());
}

void Slider::mouseMoveEvent(QMouseEvent* e)
{
	QAbstractSlider::mouseMoveEvent(e);
	if(orientation() == Qt::Horizontal)
	setValue(int(maximum() / double(width()) * e->x()) );
	else
	setValue(maximum() - int(maximum() / double(height()) * e->y()) );
	emit valueChanged(value());
}

void Slider::wheelEvent(QWheelEvent* e)
{
	QAbstractSlider::wheelEvent(e);
	emit valueChanged(value());
}


