/***************************************************************************
*   Copyright (C) 2007 by Messer                                          *
*   neomatraz@gmail.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

/**
	System tray icon module
*/

#include "tray.h"
#include <QtGui>
#include <QtCore>
#include "ui_about.h"
#include "slider.h"

Tray::Tray (QWidget *parent) : QSystemTrayIcon (parent)
{
	//trayIcon = new QSystemTrayIcon(this);
	/*trayIcon->*/setIcon(QPixmap(":/images/juk.png"));
	//trayIcon->show ();
	trayMenu = new QMenu;

	closeAct = new QAction (tr("Exit"), this);
	showAct = new QAction (tr("Show"), this);
	hideAct = new QAction (tr("Hide"), this);
	aboutAct = new QAction (tr("About player"), this);
	aboutAct->setIcon(QPixmap(":/images/about.png"));
	playAct = new QAction(tr("Play / Pause"), this);
	playAct->setIcon(QPixmap(":/images/player_play.png"));
	stopAct = new QAction(tr("Stop"), this);
	stopAct->setIcon(QPixmap(":/images/player_stop.png"));
	prev = new QAction(tr("Back"), this);
	prev->setIcon(QPixmap(":/images/player_bwd.png"));
	next = new QAction(tr("Next"), this);
	next->setIcon(QPixmap(":/images/player_end.png"));


	trayMenu->addAction (playAct);
	trayMenu->addAction (stopAct);
	trayMenu->addAction (prev);
	trayMenu->addAction (next);
	trayMenu->addSeparator();
	trayMenu->addAction (showAct);
	trayMenu->addAction (hideAct);
	trayMenu->addSeparator();
	trayMenu->addAction (aboutAct);
	trayMenu->addSeparator();
	trayMenu->addAction (closeAct);

	/*trayIcon->*/setContextMenu(trayMenu);
	connect (playAct, SIGNAL(triggered()), this, SLOT(onPlay()));
	connect (stopAct, SIGNAL(triggered()), this, SLOT(onStop()));
	connect (prev, SIGNAL(triggered()), this, SLOT(onPrev()));
	connect (next, SIGNAL(triggered()), this, SLOT(onNext()));
	connect (closeAct, SIGNAL(triggered()), this, SLOT(quitThis()));
	connect (showAct, SIGNAL(triggered()), this, SLOT(showM()));
	connect (hideAct, SIGNAL(triggered()), this, SLOT(hideM()));
	connect (/*trayIcon*/this,
		SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
	this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	connect (&mainImpl, SIGNAL(trackChanged(const QString&, const QString&,
		const QString&)),
		 this, SLOT(_setToolTip(const QString &, const QString&,
			 const QString&)));
	connect (aboutAct, SIGNAL(triggered()), this, SLOT(about()));
	connect (&mainImpl, SIGNAL(playerState(Player::State)), this,
		SLOT(setTrayIcon(Player::State)));
	mainImpl.show ();
}

void Tray::onPlay()
{
	mainImpl.playOrPause();
}

void Tray::onStop()
{
	mainImpl.stopTrack();
}

void Tray::onPrev()
{
	mainImpl.upTrack();
}

void Tray::onNext()
{
	mainImpl.downTrack();
}

void Tray::showM ()
{
	if (!mainImpl.isVisible()) mainImpl.show ();
}

void Tray::hideM ()
{
	if (mainImpl.isVisible()) mainImpl.hide ();
}


void Tray::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
if (reason == QSystemTrayIcon::Trigger)
	{
		if (Qt::LeftButton)
		{
			if (mainImpl.isVisible())
			mainImpl.hide();

			else mainImpl.show ();
		}
		if (Qt::RightButton)
		{
			return;
		}
	}

switch(mainImpl.isVisible())
{
	case true:
		showAct->setEnabled (false);
		hideAct->setEnabled (true);
		break;
	case false:
		showAct->setEnabled (true);
		hideAct->setEnabled (false);
		break;
}

}

void Tray::quitThis ()
{
//	QDir d;
//	d.rmdir("/tmp/qxplayer-lock");
	mainImpl.saveSettings();
	qApp->quit ();
}

void Tray::_setToolTip(const QString& art, const QString& alb, const QString& tit)
{
	/*trayIcon->*/setToolTip(
	"<b><center>Player</center></b><br>Artist: " +
	art + "<br>" +
	"Album: " + alb + "<br>" +
	"Title: " + tit + "<br>"
	);
}

void Tray::setTrayIcon(Player::State s)
{
	if( s == Player::Idle)
		/*trayIcon->*/setIcon(QPixmap(":/images/juk.png"));
	if (s == Player::Play)
		/*trayIcon->*/setIcon(QPixmap(":/images/play.png"));
	if (s == Player::Pause)
		/*trayIcon->*/setIcon(QPixmap(":/images/pause.png"));
}

void Tray::about()
{
	QDialog *d = new QDialog;
	Ui::About about;
	about.setupUi(d);
	d->exec();
	delete d;
}
